import { world } from "@minecraft/server";

// Funciones para manejar los datos de las celdas
export function getCellData() {
    const cellDataJson = world.getDynamicProperty("cellData");
    return JSON.parse(cellDataJson || "{}");
}

export function setCellData(cellData) {
    world.setDynamicProperty("cellData", JSON.stringify(cellData));
}

// Registrar una nueva celda
export function registerCell(cellId, position) {
    const cellData = getCellData();
    cellData[cellId] = { position, occupied: false };
    setCellData(cellData);

    return
}

// Eliminar una celda
export function deleteCell(cellId) {
    const cellData = getCellData();

    if (cellData[cellId]) {
        delete cellData[cellId];
        setCellData(cellData);
        return true;  // Indica que la celda fue eliminada exitosamente
    }

    return false;  // Indica que la celda no existía
}


// Obtener los datos de una celda específica
export function getCell(cellId) {
    const cellData = getCellData();
    return cellData[cellId] || null;
}

// Obtener los datos de una celda específica
export function getAllCells() {
    const cellData = getCellData();
    const availableCells = [];

    for (const cellId in cellData) {
        availableCells.push({ cellId, ...cellData[cellId] });
    }
    return availableCells;
}

// Obtener una celda aleatoria
export function getRandomCell() {
    const cellData = getCellData();
    const cellIds = Object.keys(cellData);

    if (cellIds.length === 0) {
        return null;
    }

    const randomIndex = Math.floor(Math.random() * cellIds.length);
    const randomCellId = cellIds[randomIndex];
    return { cellId: randomCellId, ...cellData[randomCellId] };
}

// Función para aprisionar a un jugador
export function imprisonPlayer(player) {
    var cell = getRandomCell();
    if (!cell) {
        world.sendMessage("§a No hay celdas disponibles para encarcelar")
    }
    var worldDimension = world.getDimension("overworld")
    var worldLocation = cell.position
    player.teleport(worldLocation, { dimension: worldDimension });
    player.resetLevel();
    player.setSpawnPoint({ dimension: worldDimension, x: worldLocation.x, y: worldLocation.y, z: worldLocation.z })
    player.runCommand(`tellraw @s {"rawtext":[{"text":"§bHas sido §cencarcelado en la celda §a${cell.cellId}."}]}`);
    player.runCommand("title @s title Estás Preso")

}
