import { world } from "@minecraft/server";


export function getCrimeData() {
    const crimeDataJson = world.getDynamicProperty("crimeData");
    return JSON.parse(crimeDataJson || "{}");
}

export function setCrimeData(crimeData) {
    world.setDynamicProperty("crimeData", JSON.stringify(crimeData));
}

export function registerCrime(playerName, crime, motivo, punishment) {
    const crimeData = getCrimeData();
    if (!crimeData[playerName]) {
        crimeData[playerName] = [];
    }
    crimeData[playerName].push({ crime, motivo, punishment, timestamp: new Date().toISOString() });
    setCrimeData(crimeData);
}

export function getPlayerCrimes(playerName) {
    const crimeData = getCrimeData();
    return crimeData[playerName] || [];
}

export function deleteCrime(playerName, index) {
    const crimeData = getCrimeData();
    if (crimeData[playerName]) {
        crimeData[playerName].splice(index, 1);
        setCrimeData(crimeData);
    }
}

export function clearPlayerCrimes(playerName) {
    const crimeData = getCrimeData();
    if (crimeData[playerName]) {
        delete crimeData[playerName];
        setCrimeData(crimeData);
    }
}

export function clearAllCrimes() {
    setCrimeData({});
}

export function showPlayerCrimes(playerName) {
    const crimes = getPlayerCrimes(playerName);
    crimes.forEach(crime => {
        world.sendMessage(`Criminal: ${playerName} Delito: ${crime.crime}, Castigo: ${crime.punishment}, Fecha: ${crime.timestamp}`);
    });
}