import { world, EntityComponentTypes } from "@minecraft/server";
import {
    ActionFormData,
    MessageFormData,
    ModalFormData
} from "@minecraft/server-ui";

import {
    registerCrime,
    getPlayerCrimes,
    deleteCrime,
    clearPlayerCrimes,
    clearAllCrimes,
    showPlayerCrimes
} from "./CrimeList";

import {
    registerCell,
    getCell,
    getAllCells,
    deleteCell,
    getRandomCell,
    imprisonPlayer
} from "./Cells";



// Inicializar almacenamiento global para crímenes y castigos
if (!world.getDynamicProperty("crimeData")) {
    world.setDynamicProperty("crimeData", JSON.stringify({}));
}

if (!world.getDynamicProperty("cellData")) {
    world.setDynamicProperty("cellData", JSON.stringify({}));
}



// Suscribirse al evento entityHurt
world.afterEvents.entityHurt.subscribe(event => {
    const { hurtEntity, damageSource } = event;
    playerKilled(hurtEntity, damageSource);

});



// Función para manejar comandos personalizados
world.beforeEvents.chatSend.subscribe(event => {
    const { sender, message } = event;


    if (message.startsWith("?")) {
        const args = message.split(" ");

        if (!sender.isOp()) {
            sender.runCommandAsync(`tellraw @s {"rawtext":[{"text":"§cNo tienes Permisos suficientes"}]}`)
            return
        }
        if (message === "?clearCrimes") {
            clearAllCrimes();
            sender.runCommandAsync(`tellraw @s {"rawtext":[{"text":"§cTodos los crimenes fueron eliminados!"}]}`);
        }

    }

});

world.afterEvents.itemUse.subscribe(event => {
    let sender = event.source
    if (event.itemStack.typeId === "crime:crime_menu") {

        if (!sender.isOp()) {
            sender.runCommandAsync(`tellraw @s {"rawtext":[{"text":"§cNo tienes Permisos!!!"}]}`)
            return
        }
        crimeMenuView(event.source);
    };
});

function playerCrimesView(player) {


    let listForm = new ModalFormData();
    listForm.title("Lista de Jugadores a revisar");
    listForm.submitButton("Revisar delitos")
    let playersList = []
    var playersOnline = world.getAllPlayers()
    if (!playersOnline && playersOnline.length <= 0) {
        player.runCommand("tell @s Ups ¿No hay jugadores?")
        return
    }
    playersOnline.forEach(p => {
        playersList.push(p.nameTag)
    })

    listForm.dropdown("Jugadores", playersList)

    listForm.show(player).then(r => {
        if (r.canceled) return;

        let [dropdown] = r.formValues
        var p = world.getAllPlayers()[dropdown]

        var playerName = p.nameTag

        let form = new ActionFormData();
        form.title("Delitos de " + playerName);

        var crimenes = getPlayerCrimes(playerName)
        var body = "";

        if (!crimenes || crimenes.length == 0) {
            body = "§eEste jugador no tiene ningun delito en su historial."
        }
        else {
            crimenes.forEach(r => {
                body += `§eDelito: ${r.crime} | §eMotivo: ${r.motivo} \n\n`
            })
        }

        form.body(`Delitos Totales: §a${crimenes.length} \n\n` + body)
        form.button("Ok");
        form.show(player).then(r => {

        })

    })
}

function imprisionPlayer(player) {
    let form = new ModalFormData();
    form.title("Lista de jugadores a encarcelar");
    form.submitButton("Encarcelar")
    let options = [];
    let delitos = ["Robo", "Asesinato", "Romper Reglas"]

    world.getAllPlayers().forEach(p => {
        options.push(p.nameTag)
    })

    form.dropdown("Jugador", options);
    form.dropdown("Delito", delitos);
    form.textField("Motivos", "El motivo por el encarcelamiento")
    form.textField("Castigo", "Una descripción del castigo")


    form.show(player).then(r => {
        // This will stop the code when the player closes the form
        if (r.canceled) return;

        // This will assign every input their own variable
        let [dropdown, dropdown2, textfield, textfield2] = r.formValues;

        var playerToImprision = world.getAllPlayers()[dropdown]
        var delito = delitos[dropdown2]
        var motivo = textfield
        var castigo = textfield2

        if (playerToImprision && motivo && castigo) {
            punishManualPlayer(playerToImprision, delito, motivo, castigo)
        }
        else {
            player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"§cHubo un problema al aprisionar al jugador"}]}`);
        }



        // Do something
    }).catch(e => {
        console.error(e, e.stack);
    });



}


function createCellView(player) {

    let form = new ModalFormData();
    form.title("Registrar una celda");
    form.submitButton("Guardar")

    form.textField("Numero de Celda", "Ej: 1, 2, 3")
    form.toggle("Usar ubicacion actual (tú)", false)
    form.textField("Posicion de la celda en X", " Posicion")
    form.textField("Posicion de la celda en Y", " Posicion")
    form.textField("Posicion de la celda en Z", " Posicion")



    form.show(player).then(r => {
        // This will stop the code when the player closes the form
        if (r.canceled) return;

        // This will assign every input their own variable
        let [textField, toggle, textField2, textField3, textField4] = r.formValues;

        const id = textField
        const x = toggle ? player.location.x : parseFloat(textField2)
        const y = toggle ? player.location.y : parseFloat(textField3)
        const z = toggle ? player.location.z : parseFloat(textField4)


        registerCell(id, { x, y, z });
        player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"Celda ${id} registrada exitosamente en (${x}, ${y}, ${z})."}]}`);

    })
}


function listCellsView(player) {
    let form = new ActionFormData();
    form.title("Lista de celdas")
    var cellList = ""

    var cells = getAllCells();
    cells.forEach(c => {
        cellList += `\n §eCelda Id: ${c.cellId} | §aPosicion: ${c.position.x} ${c.position.y} ${c.position.z} \n\n`
    })

    form.body("\n " + cellList)
    form.button("OK");

    form.show(player).then(r => {

    })
}

function deleteCellView(player) {
    let form = new ModalFormData();
    form.title("Eliminar una celda")
    form.submitButton("Eliminar celda")
    form.textField("Id de la Celda", "Ej: 1, 2 o 3")

    form.show(player).then(r => {
        if (r.canceled) return;

        // This will assign every input their own variable
        let [textField] = r.formValues;

        if (textField) {
            const id = textField
            var success = deleteCell(id)
            if (success) {
                player.runCommand(`tellraw @s {"rawtext":[{"text":"La celda §e${id} se ha §aeliminado"}]}`);
            }
            else {
                player.runCommand(`tellraw @s {"rawtext":[{"text":"§cHubo un problema al eliminar la celda tal vez ¿no existe?"}]}`);
            }
        }
    })
}

function crimeMenuView(player) {
    let form = new ActionFormData();
    form.title("Menú de Crimenes");
    form.body(" Elige la opción que deseas ejecutar \n\n");
    form.button("Ver los crimenes de un jugador");
    form.button("Encarcelar jugador");
    form.button("Registrar Celda");
    form.button("Lista de Celdas");
    form.button("Eliminar Celda");

    form.show(player).then(response => {
        let select = response.selection

        switch (select) {
            case 0:
                playerCrimesView(player)
                break;

            case 1:
                imprisionPlayer(player);
                break;

            case 2:
                createCellView(player);
                break;

            case 3:
                listCellsView(player);
                break;

            case 4:
                deleteCellView(player);
                break;

        }
    }).catch((e) => {
        console.error(e, e.stack);
    });


}

function playerKilled(hurtEntity, damageSource) {
    // Verificar si la entidad dañada es un jugador
    if (hurtEntity.typeId === "minecraft:player") {
        const player = hurtEntity;
        const health = player.getComponent(EntityComponentTypes.Health);

        // Verificar si la salud del jugador es 0 o menos
        if (health.currentValue <= 0) {
            // Verificar si la causa del daño es un jugador
            const killer = damageSource.damagingEntity;
            if (killer && killer.typeId === "minecraft:player") {
                const deadPlayer = player;
                const killerPlayer = killer;

                // Llamar a la función para castigar al jugador asesino
                punishKiller(killerPlayer, deadPlayer);

            }
        }
    }
}

function punishKiller(killerPlayer, deadPlayer) {

    // Vaciar el inventario del jugador asesino
    const inventory = killerPlayer.getComponent(EntityComponentTypes.Inventory).container;
    for (let i = 0; i < inventory.size; i++) {
        inventory.setItem(i, undefined);
    }

    // Registrar el crimen y el castigo
    registerCrime(killerPlayer.nameTag, "Asesinato", "Asesinó a " + deadPlayer.nameTag, "Encarcelado e inventario Decomisado");
    // Notificar al jugador asesino
    killerPlayer.runCommand(`tellraw @s {"rawtext":[{"text":"§cHas sido teletransportado y has perdido tus cosas por matar a otro jugador."}]}`);
    world.sendMessage("§b" + killerPlayer.nameTag + " §fha sido §cencarcelado §bpor §aasesinar §ba §a" + deadPlayer.nameTag)
    imprisonPlayer(killerPlayer);
}

function punishManualPlayer(player, crimen, motivo, castigo) {

    // Vaciar el inventario del jugador asesino
    const inventory = player.getComponent(EntityComponentTypes.Inventory).container;
    for (let i = 0; i < inventory.size; i++) {
        inventory.setItem(i, undefined);
    }

    // Registrar el crimen y el castigo
    registerCrime(player.nameTag, crimen, motivo, castigo);
    // Notificar al jugador asesino

    world.sendMessage("§b" + player.nameTag + " §fha sido §cencarcelado §bpor §a" + crimen)
    imprisonPlayer(player);
}





